///File : ConfigManager.cpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "ConfigManager.hpp"
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//------------------------------- ConfigManager --------------------------------
//------------------------------------------------------------------------------

ConfigManager::ConfigManager()
{
    ClearAll();
}
//------------------------------------------------------------------------------
void ConfigManager::operator=(const ConfigManager &src) // Copie des données uniquement
{
    ClearAll();
    m_DBbool = src.m_DBbool;
    m_DBint = src.m_DBint;
    m_DBdouble = src.m_DBdouble;
    m_DBstring = src.m_DBstring;
    m_isModified = src.m_isModified;
}
//------------------------------------------------------------------------------
void ConfigManager::LoadFromElement(ticpp::Element *root)
{
    ClearAll();
    m_isOk = true;

    ticpp::Element *typeSection;
    ticpp::Element *param;

    if(root->FirstChild("version")->ToElement()->GetText(false) == "1")
    {
        typeSection = root->FirstChildElement("bool", false);
        if(typeSection)
        {
            param = typeSection->FirstChildElement(false);
            while(param != NULL)
            {
                m_DBbool[param->Value()] = stringToNb<bool>(param->GetAttribute("value"));
                param = param->NextSiblingElement(false);
            }
        }

        typeSection = root->FirstChildElement("int", false);
        if(typeSection)
        {
            param = typeSection->FirstChildElement(false);
            while(param != NULL)
            {
                m_DBint[param->Value()] = stringToNb<int>(param->GetAttribute("value"));
                param = param->NextSiblingElement(false);
            }
        }

        typeSection = root->FirstChildElement("double", false);
        if(typeSection)
        {
            param = typeSection->FirstChildElement(false);
            while(param != NULL)
            {
                m_DBdouble[param->Value()] = stringToNb<double>(param->GetAttribute("value"));
                param = param->NextSiblingElement(false);
            }
        }

        typeSection = root->FirstChildElement("string", false);
        if(typeSection)
        {
            param = typeSection->FirstChildElement(false);
            while(param != NULL)
            {
                m_DBstring[param->Value()] = param->GetAttribute("value");
                param = param->NextSiblingElement(false);
            }
        }
    }
    else
    {
        throw ticpp::Exception("Bad config version !");
    }
}
//------------------------------------------------------------------------------
/*void ConfigManager::LoadFromFile(std::string filename)
{
    if( filename == "")
    {
        filename = m_filename;
    }
    else
    {
        m_filename = filename;
    }

    ticpp::Document myDoc(filename);
    myDoc.LoadFile();

    LoadFromElement(myDoc.FirstChild("config")->ToElement());
}*/
//------------------------------------------------------------------------------
void ConfigManager::ClearAll()
{
	m_DBbool.clear();
	m_DBdouble.clear();
	m_DBint.clear();
	m_DBstring.clear();

    m_isModified = false;
    m_isOk = true; // On peut commencer à l'utiliser vide...
}
//------------------------------------------------------------------------------
// Sauvegarde
ticpp::Element ConfigManager::SaveToElement()
{
    ThrowNotOkIfNeeded();
    m_isModified = false;

    ticpp::Element root("config");

    ticpp::Element *typeSection;
    ticpp::Element *param;

    ticpp::Element Version("version");
    Version.SetText("1");
    root.InsertEndChild(Version);

    // bool
    if(!m_DBbool.empty())
    {
        typeSection = new ticpp::Element("bool");
        std::map<std::string, bool>::reverse_iterator boolRit;
        for ( boolRit = m_DBbool.rbegin() ; boolRit != m_DBbool.rend(); boolRit++ )
        {
            param = new ticpp::Element(boolRit->first);
            param->SetAttribute("value", nbToString(m_DBbool.at(boolRit->first)));
            typeSection->InsertEndChild(*param);
            delete param;
        }
        root.InsertEndChild(*typeSection);
        delete typeSection;
    }

    // int
    if(!m_DBint.empty())
    {
        typeSection = new ticpp::Element("int");
        std::map<std::string, int>::reverse_iterator intRit;
        for ( intRit = m_DBint.rbegin() ; intRit != m_DBint.rend(); intRit++ )
        {
            param = new ticpp::Element(intRit->first);
            param->SetAttribute("value", nbToString(m_DBint.at(intRit->first)));
            typeSection->InsertEndChild(*param);
            delete param;
        }
        root.InsertEndChild(*typeSection);
        delete typeSection;
    }

    // double
    if(!m_DBdouble.empty())
    {
        typeSection = new ticpp::Element("double");
        std::map<std::string, double>::reverse_iterator doubleRit;
        for ( doubleRit = m_DBdouble.rbegin() ; doubleRit != m_DBdouble.rend();doubleRit++ )
        {
            param = new ticpp::Element(doubleRit->first);
            param->SetAttribute("value", nbToString(m_DBdouble.at(doubleRit->first)));
            typeSection->InsertEndChild(*param);
            delete param;
        }
        root.InsertEndChild(*typeSection);
        delete typeSection;
    }

    // string
    if(!m_DBstring.empty())
    {
        typeSection = new ticpp::Element("string");
        std::map<std::string, std::string>::reverse_iterator stringRit;
        for ( stringRit = m_DBstring.rbegin() ; stringRit != m_DBstring.rend(); stringRit++ )
        {
            param = new ticpp::Element(stringRit->first);
            param->SetAttribute("value", m_DBstring.at(stringRit->first));
            typeSection->InsertEndChild(*param);
            delete param;
        }
        root.InsertEndChild(*typeSection);
        delete typeSection;
    }

    return root;
}
//------------------------------------------------------------------------------
/*void ConfigManager::SaveToFile(std::string filename)
{
    ThrowNotOkIfNeeded();

    if( filename == "")
    {
        filename = m_filename;
    }
    else
    {
        m_filename = filename;
    }

    ticpp::Document myDoc(filename);

    ticpp::Declaration myDeclaration("1.0","UTF-8","no");

    myDoc.InsertEndChild(myDeclaration);
    ticpp::Element root = SaveToElement();
    myDoc.InsertEndChild(root);

    myDoc.SaveFile();
}*/
//------------------------------------------------------------------------------

// Get
bool ConfigManager::GetBool(const std::string &key) const
{
    ThrowNotOkIfNeeded();
    if( ExistsBool(key) )
    {
        return m_DBbool.at(key);
    }
    else
    {
        std::string msg = "No param with this key in the config db !\n";
        msg += "Key : \"";
        msg += key;
        msg += "\".";
        throw ticpp::Exception(msg.c_str());
    }
}
//------------------------------------------------------------------------------
int ConfigManager::GetInt(const std::string &key) const
{
    ThrowNotOkIfNeeded();
    if( ExistsInt(key) )
    {
        return m_DBint.at(key);
    }
    else
    {
        std::string msg = "No param with this key in the config db !\n";
        msg += "Key : \"";
        msg += key;
        msg += "\".";
        throw ticpp::Exception(msg.c_str());
    }
}
//------------------------------------------------------------------------------
double ConfigManager::GetDouble(const std::string &key) const
{
    ThrowNotOkIfNeeded();
    if( ExistsDouble(key) )
    {
        return m_DBdouble.at(key);
    }
    else
    {
        std::string msg = "No param with this key in the config db !\n";
        msg += "Key : \"";
        msg += key;
        msg += "\".";
        throw ticpp::Exception(msg.c_str());
    }
}
//------------------------------------------------------------------------------
std::string ConfigManager::GetString(const std::string &key) const
{
    ThrowNotOkIfNeeded();
    if( ExistsString(key) )
    {
        return m_DBstring.at(key);
    }
    else
    {
        std::string msg = "No param with this key in the config db !\n";
        msg += "Key : \"";
        msg += key;
        msg += "\".";
        throw ticpp::Exception(msg.c_str());
    }
}
//------------------------------------------------------------------------------
StructStdFont ConfigManager::GetFont(const std::string &key) const
{
    StructStdFont tmp;
    tmp.bold = GetBool(std::string("font_") + key + std::string("_isBold"));
    tmp.italic = GetBool(std::string("font_") + key + std::string("_isItalic"));
    tmp.underligned = GetBool(std::string("font_") + key + std::string("_isUnderligned"));
    tmp.faceName = GetString(std::string("font_") + key + std::string("_faceName"));
    tmp.pointSize = GetInt(std::string("font_") + key + std::string("_pointSize"));
    return tmp;
}
//------------------------------------------------------------------------------
StructStdColor ConfigManager::GetColor(const std::string &key) const
{
    StructStdColor tmp;
    tmp.r = GetInt(std::string("color_") + key + std::string("_r"));
    tmp.g = GetInt(std::string("color_") + key + std::string("_g"));
    tmp.b = GetInt(std::string("color_") + key + std::string("_b"));
    return tmp;
}
//------------------------------------------------------------------------------
StructStdColorFont ConfigManager::GetColorFont(const std::string &key) const
{
    StructStdColorFont tmp;
    tmp.color = GetColor(key);
    tmp.font = GetFont(key);
    return tmp;
}
//------------------------------------------------------------------------------
// Set
void ConfigManager::SetBool(const std::string &key, const bool &value)
{
    ThrowNotOkIfNeeded();
    m_isModified = true;
    m_DBbool[key] = value;
}
//------------------------------------------------------------------------------
void ConfigManager::SetInt(const std::string &key, const int &value)
{
    ThrowNotOkIfNeeded();
    m_isModified = true;
    m_DBint[key] = value;
}
//------------------------------------------------------------------------------
void ConfigManager::SetDouble(const std::string &key, const double &value)
{
    ThrowNotOkIfNeeded();
    m_isModified = true;
    m_DBdouble[key] = value;
}
//------------------------------------------------------------------------------
void ConfigManager::SetString(const std::string &key, const std::string &value)
{
    ThrowNotOkIfNeeded();
    m_isModified = true;
    m_DBstring[key] = value;
}
//------------------------------------------------------------------------------
void ConfigManager::SetFont(const std::string &key, const StructStdFont &value)
{
    SetBool(std::string("font_") + key + std::string("_isBold"), value.bold);
    SetBool(std::string("font_") + key + std::string("_isItalic"), value.italic);
    SetBool(std::string("font_") + key + std::string("_isUnderligned"), value.underligned);
    SetString(std::string("font_") + key + std::string("_faceName"), value.faceName);
    SetInt(std::string("font_") + key + std::string("_pointSize"), value.pointSize);
}
//------------------------------------------------------------------------------
void ConfigManager::SetColor(const std::string &key, const StructStdColor &value)
{
    SetInt(std::string("color_") + key + std::string("_r"), value.r );
    SetInt(std::string("color_") + key + std::string("_g"), value.g );
    SetInt(std::string("color_") + key + std::string("_b"), value.b );
}
//------------------------------------------------------------------------------
void ConfigManager::SetColorFont(const std::string &key, const StructStdColorFont &value)
{
    SetColor(key, value.color );
    SetFont(key, value.font );
}
//------------------------------------------------------------------------------
// Remove
void ConfigManager::RemoveBool(const std::string &key)
{
    ThrowNotOkIfNeeded();
	if( ExistsBool(key))
	{
		m_DBbool.erase(key);
		m_isModified = true;
	}
    else
    {
        std::string msg = "Remove : No param with this key in the config db !\n";
        msg += "Key : \"";
        msg += key;
        msg += "\".";
        throw ticpp::Exception(msg.c_str());
    }
}

//------------------------------------------------------------------------------
void ConfigManager::RemoveInt(const std::string &key)
{
    ThrowNotOkIfNeeded();
	if( ExistsInt(key))
	{
		m_DBint.erase(key);
		m_isModified = true;
	}
    else
    {
        std::string msg = "Remove : No param with this key in the config db !\n";
        msg += "Key : \"";
        msg += key;
        msg += "\".";
        throw ticpp::Exception(msg.c_str());
    }
}
//------------------------------------------------------------------------------
void ConfigManager::RemoveDouble(const std::string &key)
{
    ThrowNotOkIfNeeded();
	if( ExistsDouble(key))
	{
		m_DBdouble.erase(key);
		m_isModified = true;
	}
    else
    {
        std::string msg = "Remove : No param with this key in the config db !\n";
        msg += "Key : \"";
        msg += key;
        msg += "\".";
        throw ticpp::Exception(msg.c_str());
    }
}
//------------------------------------------------------------------------------
void ConfigManager::RemoveString(const std::string &key)
{
    ThrowNotOkIfNeeded();
	if( ExistsString(key))
	{
		m_DBstring.erase(key);
		m_isModified = true;
	}
    else
    {
        std::string msg = "Remove : No param with this key in the config db !\n";
        msg += "Key : \"";
        msg += key;
        msg += "\".";
        throw ticpp::Exception(msg.c_str());
    }
}
//------------------------------------------------------------------------------
void ConfigManager::RemoveFont(const std::string &key)
{
    RemoveBool(std::string("font_") + key + std::string("_isBold"));
    RemoveBool(std::string("font_") + key + std::string("_isItalic"));
    RemoveBool(std::string("font_") + key + std::string("_isUnderligned"));
    RemoveString(std::string("font_") + key + std::string("_faceName"));
    RemoveInt(std::string("font_") + key + std::string("_pointSize"));
}
//------------------------------------------------------------------------------
void ConfigManager::RemoveColor(const std::string &key)
{
    RemoveInt(std::string("color_") + key + std::string("_r"));
    RemoveInt(std::string("color_") + key + std::string("_g"));
    RemoveInt(std::string("color_") + key + std::string("_b"));
}
//------------------------------------------------------------------------------
void ConfigManager::RemoveColorFont(const std::string &key)
{
    RemoveColor(key);
    RemoveFont(key);
}
//------------------------------------------------------------------------------
// Test
bool ConfigManager::ExistsBool(const std::string &key) const
{
    ThrowNotOkIfNeeded();
    return (m_DBbool.find(key) != m_DBbool.end());
}
//------------------------------------------------------------------------------
bool ConfigManager::ExistsInt(const std::string &key) const
{
    ThrowNotOkIfNeeded();
    return (m_DBint.find(key) != m_DBint.end());
}
//------------------------------------------------------------------------------
bool ConfigManager::ExistsDouble(const std::string &key) const
{
    ThrowNotOkIfNeeded();
    return (m_DBdouble.find(key) != m_DBdouble.end());
}
//------------------------------------------------------------------------------
bool ConfigManager::ExistsString(const std::string &key) const
{
    ThrowNotOkIfNeeded();
    return (m_DBstring.find(key) != m_DBstring.end());
}
//------------------------------------------------------------------------------

bool ConfigManager::ExistsInCfg(const std::string &key) const
{
    ThrowNotOkIfNeeded();
    return ( ExistsBool(key) || ExistsDouble(key) || ExistsInt(key) || ExistsString(key));
}
//------------------------------------------------------------------------------

// Flags
bool ConfigManager::IsOk() const
{
    return m_isOk;
}
//------------------------------------------------------------------------------
bool ConfigManager::IsModified() const
{
    ThrowNotOkIfNeeded();
    return m_isModified;
}
//------------------------------------------------------------------------------
void ConfigManager::SetModified(const bool &isModified)
{
    ThrowNotOkIfNeeded();
    m_isModified = isModified;
}
//------------------------------------------------------------------------------

// Filename
/*std::string ConfigManager::GetFilename() const
{
    return m_filename;
}
//------------------------------------------------------------------------------
void ConfigManager::SetFilename(const std::string &filename)
{
    m_filename = filename;
    m_isModified = true;
}
//------------------------------------------------------------------------------
*/
void ConfigManager::ThrowNotOkIfNeeded() const
{
    if( !IsOk() )
    {
        throw ticpp::Exception("The config db is not ready !");
    }
}
//------------------------------------------------------------------------------
//------------------------------- class GeneralCfg -----------------------------
//------------------------------------------------------------------------------

GeneralCfg::GeneralCfg()
{
    m_isModified = false;
    #if defined USE_MULTIPLE_CONFIG
    m_currentCfgPos = 0;
    m_defaultCfgPos = 0;
    #endif
}
//------------------------------------------------------------------------------
void GeneralCfg::SaveToFile(const std::string &filename)
{
    if(filename != "")
    {
        SetFilename(filename);
    }
    m_isModified = false;

    ticpp::Document myDoc(m_filename);

    ticpp::Declaration myDeclaration("1.0","UTF-8","no");

    myDoc.InsertEndChild(myDeclaration);

    ticpp::Element root("globalConfigs");

    #if defined USE_MULTIPLE_CONFIG
        // Default
        ticpp::Element defaultCfg("defaultCfg");
        defaultCfg.SetAttribute("name", m_cfgTbl[m_defaultCfgPos].generalCfg.GetString("cfgName"));
        root.InsertEndChild(defaultCfg);

        // Data
        for(int i = 0; i < m_cfgTbl.size(); i++)
        {
            ticpp::Element globalConfig("globalConfig");

            ticpp::Element defaultDevisCfgElement = m_cfgTbl[i].defaultDevisCfg.SaveToElement();
            ticpp::Element defaultDevisElement("defaultDevisCfg");
            defaultDevisElement.InsertEndChild(defaultDevisCfgElement);
            globalConfig.InsertEndChild(defaultDevisElement);

            ticpp::Element generalElement("generalCfg");
            ticpp::Element generalCfgElement = m_cfgTbl[i].generalCfg.SaveToElement();
            generalElement.InsertEndChild(generalCfgElement);
            globalConfig.InsertEndChild(generalElement);

            root.InsertEndChild(globalConfig);
        }
    #else
        // Default ==> Meme pour simple config pour compatibilité...
        ticpp::Element defaultCfg("defaultCfg");
        defaultCfg.SetAttribute("name", m_cfg.generalCfg.GetString("cfgName"));
        root.InsertEndChild(defaultCfg);

        ticpp::Element globalConfig("globalConfig");

        ticpp::Element defaultDevisCfgElement = m_cfg.defaultDevisCfg.SaveToElement();
        ticpp::Element defaultDevisElement("defaultDevisCfg");
        defaultDevisElement.InsertEndChild(defaultDevisCfgElement);
        globalConfig.InsertEndChild(defaultDevisElement);

        ticpp::Element generalElement("generalCfg");
        ticpp::Element generalCfgElement = m_cfg.generalCfg.SaveToElement();
        generalElement.InsertEndChild(generalCfgElement);
        globalConfig.InsertEndChild(generalElement);

        root.InsertEndChild(globalConfig);
    #endif

    myDoc.InsertEndChild(root);

    myDoc.SaveFile();
}
//------------------------------------------------------------------------------
void GeneralCfg::LoadFromFile(const std::string &filename)
{
    if(filename != "")
    {
        SetFilename(filename);
    }
    m_isModified = false;

    ticpp::Document myDoc(m_filename);
    myDoc.LoadFile();

    ticpp::Element *root = myDoc.FirstChildElement("globalConfigs");

    #if defined USE_MULTIPLE_CONFIG
        // Default
        ticpp::Element *defaultCfgNameElement = root->FirstChildElement("defaultCfg", false);
        std::string defaultCfgName = "";
        if(defaultCfgNameElement != NULL) // Peut ne pas exister => Pas grave.
        {
            defaultCfgName = defaultCfgNameElement->GetAttribute("name");
        }

        // Data
        bool defaultCfgFound = false;

        ticpp::Element *globalConfig = root->FirstChildElement("globalConfig");
        while(globalConfig != NULL)
        {
            m_cfgTbl.resize(m_cfgTbl.size() + 1);

            m_cfgTbl[m_cfgTbl.size() - 1].defaultDevisCfg.LoadFromElement(globalConfig->FirstChild("defaultDevisCfg")->FirstChildElement("config"));
            m_cfgTbl[m_cfgTbl.size() - 1].generalCfg.LoadFromElement(globalConfig->FirstChild("generalCfg")->FirstChildElement("config"));

            if(m_cfgTbl[m_cfgTbl.size() - 1].generalCfg.GetString("cfgName") == defaultCfgName)
            {
                m_defaultCfgPos = m_cfgTbl.size() - 1;
                m_currentCfgPos = m_defaultCfgPos;
                defaultCfgFound = true;
            }

            globalConfig = globalConfig->NextSiblingElement("globalConfig", false);
        }

        if(!defaultCfgFound)
        {
            m_defaultCfgPos = 0;
            m_currentCfgPos = m_defaultCfgPos;
            m_isModified = true; // Modifié : la "default cfg" n'existe pas.
        }
    #else
        ticpp::Element *globalConfig = root->FirstChildElement("globalConfig");
        m_cfg.defaultDevisCfg.LoadFromElement(globalConfig->FirstChild("defaultDevisCfg")->FirstChildElement("config"));
        m_cfg.generalCfg.LoadFromElement(globalConfig->FirstChild("generalCfg")->FirstChildElement("config"));
    #endif
}
//------------------------------------------------------------------------------
std::string GeneralCfg::GetFilename() const
{
    return m_filename;
}
//------------------------------------------------------------------------------
void GeneralCfg::SetFilename(const std::string &filename)
{
    if(m_filename != filename)
    {
        m_filename = filename;
        m_isModified = true;
    }
}
//------------------------------------------------------------------------------
bool GeneralCfg::IsModified() const
{
    #if defined USE_MULTIPLE_CONFIG
    if(m_isModified)
    {
        return true;
    }

    for(int i = 0; i < m_cfgTbl.size(); i++)
    {
        if(m_cfgTbl[i].generalCfg.IsModified() || m_cfgTbl[i].defaultDevisCfg.IsModified())
        {
            return true;
        }
    }

    return false;
    #else
    return ( m_isModified || m_cfg.generalCfg.IsModified() || m_cfg.defaultDevisCfg.IsModified());
    #endif
}
//------------------------------------------------------------------------------
ConfigManager& GeneralCfg::GetGeneralCfg()
{
    #if defined USE_MULTIPLE_CONFIG
    return m_cfgTbl[m_currentCfgPos].generalCfg;
    #else
    return m_cfg.generalCfg;
    #endif
}
//------------------------------------------------------------------------------
ConfigManager& GeneralCfg::GetDefaultDevisCfg()
{
    #if defined USE_MULTIPLE_CONFIG
    return m_cfgTbl[m_currentCfgPos].defaultDevisCfg;
    #else
    return m_cfg.defaultDevisCfg;
    #endif
}
//------------------------------------------------------------------------------
#if defined USE_MULTIPLE_CONFIG
//------------------------------------------------------------------------------
int GeneralCfg::GetCurrentCfgPos() const
{
    return m_currentCfgPos;
}
//------------------------------------------------------------------------------
int GeneralCfg::GetCfgCount() const
{
    return m_cfgTbl.size();
}
//------------------------------------------------------------------------------
void GeneralCfg::SelectCfg(const int &pos)
{
    if(pos >= 0 && pos < m_cfgTbl.size())
    {
        m_currentCfgPos = pos;
    }
    else
    {
        throw ticpp::Exception("SelectCfg : pos negative / too big !");
    }
}
//------------------------------------------------------------------------------
void GeneralCfg::RemoveCfg(const int &pos)
{
    if(pos >= 0 && pos < m_cfgTbl.size())
    {
        if(m_cfgTbl.size() > 1)
        {
            if(pos != m_currentCfgPos)
            {
                if(m_currentCfgPos > pos) // SI la m_currentCfgPos est après la pos à supprimer, il faut la décaler...
                {
                    m_currentCfgPos--;
                }

                if(pos == m_defaultCfgPos) // SI on supprime la config par default
                {
                    m_defaultCfgPos = 0;
                }
                else if(m_defaultCfgPos > pos) // SI la m_currentCfgPos est après la pos à supprimer, il faut la décaler...
                {
                    m_defaultCfgPos--;
                }

                m_cfgTbl.erase(m_cfgTbl.begin() + pos);
                m_isModified = true;
            }
            else
            {
                throw ticpp::Exception("RemoveCfg : can't remove current config !");
            }
        }
        else
        {
            throw ticpp::Exception("RemoveCfg : it left only one cfg, can't remove it !");
        }
    }
    else
    {
        throw ticpp::Exception("RemoveCfg : pos negative / too big !");
    }
}
//------------------------------------------------------------------------------
void GeneralCfg::CopyCfg(const int &pos)
{
    if(pos >= 0 && pos < m_cfgTbl.size())
    {
        m_cfgTbl.resize(m_cfgTbl.size() + 1);
        m_cfgTbl[m_cfgTbl.size() - 1] = m_cfgTbl[pos];
        m_isModified = true;
    }
    else
    {
        throw ticpp::Exception("CopyCfg : pos negative / too big !");
    }
}
//------------------------------------------------------------------------------
std::vector<std::string> GeneralCfg::GetCfgTbl() const
{
    std::vector<std::string> tbl;
    tbl.resize(m_cfgTbl.size());

    for(int i = 0; i < m_cfgTbl.size(); i++)
    {
        tbl[i] = m_cfgTbl[i].generalCfg.GetString("cfgName");
    }

    return tbl;
}
//------------------------------------------------------------------------------
int GeneralCfg::GetDefaultCfgPos() const
{
    return m_defaultCfgPos;
}
//------------------------------------------------------------------------------
void GeneralCfg::SetDefaultCfgPos(const int &pos)
{
    if(pos >= 0 && pos < m_cfgTbl.size())
    {
        m_defaultCfgPos = pos;
        m_isModified = true;
    }
    else
    {
        throw ticpp::Exception("SetDefaultCfgPos : pos negative / too big !");
    }
}
//------------------------------------------------------------------------------

#endif
