///File : ItemSectionDBManager.hpp
//------------------------------------------------------------------------------
/*
    Copyright (C) 2009 Colin GILLE / congelli501

    This file is part of Devis Maker.
    Devis Maker is sold under Devis Maker license (read COPYING file)
*/
//------------------------------------------------------------------------------
#if not defined ITEM_ROOM_DB_MANAGER
#define ITEM_ROOM_DB_MANAGER
//------------------------------------------------------------------------------
#include <string>
#include <iostream>
#include <vector>
#include <map>
//------------------------------------------------------------------------------
#include "ItemSectionDBBase.hpp"
//------------------------------------------------------------------------------
#include "../config.hpp"
//------------------------------------------------------------------------------
#define BACKUP_NB 20

enum
{
    BACKUP_ITEM_DB_MODIFIED,
    BACKUP_SECTION_DB_MODIFIED,
    BACKUP_ITEM_DB_ADDED,
    BACKUP_SECTION_DB_ADDED,
    BACKUP_ITEM_DB_REMOVED,
    BACKUP_SECTION_DB_REMOVED,
    BACKUP_NEW
};
//------------------------------------------------------------------------------
// Backup
struct StructBackup
{
    // DB
    std::vector<std::string> m_sectionDB;
    std::map<std::string, itemRef> m_itemDB;

    // Flags
    bool m_isOk;
    bool m_isSectionsModified, m_isItemsModified;

    // Type
    int type;
    std::string infoStr;
};

class ItemSectionDBManager : public ItemSectionDBBase
{
public:
    // Init
    ItemSectionDBManager(const std::string &sectionDBPath, const std::string &itemDBPath);

    // Modif - ItemDB
    bool ItemDBAdd(const std::string &itemID);
    bool ItemDBSetRef(const std::string &itemID, const std::string &newItemID);
    void ItemDBSetName(const std::string &itemID, const std::string &name);
    void ItemDBSetPriceHT(const std::string &itemID, const double &priceHT);
    void ItemDBSetTaxe(const std::string &itemID, const double &taxe);
    void ItemDBSetEcoPart(const std::string &itemID, const double &ecoPart);
    void ItemDBRemove(const std::string &itemID);

    // Modif - SectionDB
    void SectionDBAdd(const std::string &sectionID);
    void SectionDBSetName(const std::string &sectionID, const std::string &name);
    void SectionDBRemove(const std::string &sectionID);

    // Sauvegarde
    void SaveDBs();

    // Undo / Redo
    bool BackupCanGoTo(const int &pos) const;
    void BackupSaveCurrent(const int &type, const std::string &infoStr = "");
    void BackupLoad(const int &pos);
    int BackupGetType(const int &pos) const;
    std::string BackupGetInfoStr(const int &pos) const;


protected:

    // Filename
    std::string m_SectionDBPath, m_ItemDBPath;

	// Backup
    std::vector<StructBackup> m_backup;
    int m_backupPos; // La position actuelle dans le tableau...
};

#endif //ITEM_ROOM_DB_MANAGER
