///File : PanelRIEdit.cpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "MainPanel.hpp"

//------------------------------------------------------------------------------
//-                               PanelItems                                   -
//------------------------------------------------------------------------------
MainPanel::MainPanel( wxWindow* parent, wxWindowID id, ItemSectionDBManager *IRDBManager, bool isItems, const wxPoint& pos, const wxSize& size, long style ) : wxPanel( parent, id, pos, size, style )
{
    // Flags
    m_isItems = isItems;

    // Devis Manager
    m_IRDBManager = IRDBManager;

	// Controls
        // Search
	wxBoxSizer *MainSizer = new wxBoxSizer( wxVERTICAL );

    wxBoxSizer *SearchSizer = new wxBoxSizer( wxHORIZONTAL );

    SearchSizer->Add(new wxStaticText(this, wxID_ANY, _("Search :")), 0, wxALIGN_CENTER | wxALL, 5);

	m_TextSearch = new wxTextCtrl(this, wxID_ANY, _T(""));
	SearchSizer->Add(m_TextSearch, 1, wxALL, 5);

	MainSizer->Add(SearchSizer, 0, wxEXPAND);


        // List
    wxBoxSizer *ListBtSizer = new wxBoxSizer( wxHORIZONTAL );

    long tableStyle = wxLC_REPORT | wxLC_SINGLE_SEL;
    if( m_isItems )
    {
        tableStyle = wxLC_REPORT; // Multi selection si item
    }
    m_Table = new wxListView(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, tableStyle );
    ListBtSizer->Add(m_Table, 1, wxEXPAND, 0);

        // Bts
    wxBoxSizer *BtSizer = new wxBoxSizer( wxVERTICAL );
    m_BtAdd = new wxButton(this, wxID_ADD);
    BtSizer->Add(m_BtAdd, 1, wxALL, 5);

    m_BtRm = new wxButton(this, wxID_REMOVE);
    BtSizer->Add(m_BtRm, 1, wxALL, 5);

    m_BtEdit = new wxButton(this, wxID_EDIT);
    BtSizer->Add(m_BtEdit, 1, wxALL, 5);

    ListBtSizer->Add(BtSizer);

    MainSizer->Add(ListBtSizer, 1, wxEXPAND);

	SetSizer(MainSizer);

	// Menu
	if(isItems) // Si on traite les items
	{
	    m_PopupMenu.Append(Id_RefEdit, _("Reference"));
        m_PopupMenu.Append(Id_NameEdit, _("Name"));
        m_PopupMenu.Append(Id_PriceHTEdit, _("Price"));
        m_PopupMenu.Append(Id_TaxeEdit, _("Taxe"));
        m_PopupMenu.Append(Id_EcoPartEdit, _("Eco Part"));

        Connect( Id_RefEdit, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainPanel::OnRefEdit));
        Connect( Id_NameEdit, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainPanel::OnNameEdit));
        Connect( Id_PriceHTEdit, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainPanel::OnPriceHTEdit));
        Connect( Id_TaxeEdit, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainPanel::OnTaxeEdit));
        Connect( Id_EcoPartEdit, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(MainPanel::OnEcoPartEdit));
	}

	// Events
    m_BtAdd->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainPanel::OnAdd), NULL, this);
    m_BtRm->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainPanel::OnRm), NULL, this);
    m_BtEdit->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxEventHandler(MainPanel::OnEdit), NULL, this);

    m_Table->Connect( wxEVT_LEFT_DCLICK, wxEventHandler(MainPanel::OnEdit), NULL, this);
    m_Table->Connect( wxEVT_RIGHT_DOWN, wxEventHandler(MainPanel::OnEdit), NULL, this);
    m_Table->Connect( wxEVT_KEY_DOWN, wxKeyEventHandler(MainPanel::OnListKeyDown), NULL, this);

    m_TextSearch->Connect( wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler(MainPanel::OnSearchText), NULL, this);

    // Table
    FeedTbl();
}
//------------------------------------------------------------------------------

void MainPanel::OnAdd(wxCommandEvent& event)
{
    wxString newName;
    if(m_isItems) // Si on traite les items dans ce panel
    {
        newName = wxGetTextFromUser(_("Enter the item ref :"), _("New item"), _T(""), this);
        if(newName != _T(""))
        {
            if(!m_IRDBManager->ItemExists(StrWxToStd(newName))) // Si l'item n'xiste pas déjà...
            {
                m_IRDBManager->ItemDBAdd(StrWxToStd(newName));
                FeedTbl();

                if(m_Table->GetItemCount() > 0 && m_Table->FindItem(-1, newName, true) != -1)
                {
                    DeselectAll();
                    m_Table->Select(m_Table->FindItem(-1, newName, true)); // On met le dernier item (séléctioné)

                    wxCommandEvent tmpEvt;
                    OnNameEdit(tmpEvt);
                    OnPriceHTEdit(tmpEvt);
                }
            }
            else
            {
                wxMessageBox(_("This item already exists !"), _("New item"), wxICON_EXCLAMATION);
            }
        }
    }
    else
    {
        newName = wxGetTextFromUser(_("Enter the section name :"), _("New section"), _T(""), this);
        std::string sectionId = StrWxToStd(newName);
        if(newName != _T(""))
        {
            if(!m_IRDBManager->SectionExists(sectionId))// Si l'item n'xiste pas déjà...
            {
                m_IRDBManager->SectionDBAdd(sectionId);
                FeedTbl();
            }
            else
            {
                wxMessageBox(_("This section already exists !"), _("New item"), wxICON_EXCLAMATION);
            }
        }
    }
}
//------------------------------------------------------------------------------
std::vector<std::string> MainPanel::GetSelectedRefList() const
{
    std::vector<std::string> refList;

    // Cette fonctions permet de prendre toute la liste des ref séléctionnée avant que la suppression / l'edition ne commence...
    for(long selIndex = m_Table->GetFirstSelected() ; selIndex != wxNOT_FOUND; selIndex = m_Table->GetNextSelected(selIndex))
    {
        std::string ref = StrWxToStd(m_Table->GetItemText(selIndex));

        bool isOk;
        if( m_isItems )
        {
            isOk = m_IRDBManager->ItemExists(ref);
        }
        else
        {
            isOk = m_IRDBManager->SectionExists(ref);
        }

        if(isOk) // Si l'item existe...
        {
            refList.push_back(ref);
        }
    }

    return refList;
}
//------------------------------------------------------------------------------
void MainPanel::DeselectAll()
{
    for(long selIndex = m_Table->GetFirstSelected() ; selIndex != wxNOT_FOUND; selIndex = m_Table->GetNextSelected(selIndex))
    {
        m_Table->Select(selIndex, false);
    }
}
//------------------------------------------------------------------------------
void MainPanel::SelectAll()
{
    for(long selIndex = 0 ; selIndex < m_Table->GetItemCount(); selIndex++)
    {
        m_Table->Select(selIndex, true);
    }
}
//------------------------------------------------------------------------------
void MainPanel::OnRm(wxCommandEvent& event)
{
    std::vector<std::string> refList = GetSelectedRefList();

    // Items
    if(m_isItems && RefWarningMsgBox()) // Si on traite les items dans ce panel
    {
        for(int i = 0; i < refList.size(); i++)
        {
            m_IRDBManager->ItemDBRemove(refList[i]);
        }
        FeedTbl();
    }
    else // Suppression d'unse section...
    {
        for(int i = 0; i < refList.size(); i++)
        {
            m_IRDBManager->SectionDBRemove(refList[i]);
        }
        FeedTbl();
    }

    DeselectAll();
    if(m_Table->GetItemCount() > 0)
    {
        m_Table->Select(0);
    }
}
//------------------------------------------------------------------------------
void MainPanel::OnListKeyDown(wxKeyEvent& event)
{
    if(event.GetKeyCode() == WXK_DELETE)
    {
        wxCommandEvent tmp;
        OnRm(tmp);
    }

    event.Skip();
}
//------------------------------------------------------------------------------
void MainPanel::OnEdit(wxEvent& event)
{
    if(m_isItems) // Si on traite les items dans ce panel
    {
        PopupMenu(&m_PopupMenu);
    }
    else
    {
        long selIndex = m_Table->GetFirstSelected();

        if(m_Table->GetFirstSelected() != -1)
        {
            std::string sectionId = StrWxToStd(m_Table->GetItemText(m_Table->GetFirstSelected()));
            if(m_IRDBManager->SectionExists(sectionId))
            {
                wxString newName = wxGetTextFromUser(_("Enter the new section name :"), _("Section edit"), m_Table->GetItemText(m_Table->GetFirstSelected()), this);

                if(newName != _T("")) // Si l'item existe...
                {
                    m_IRDBManager->SectionDBSetName(sectionId, StrWxToStd(newName));
                    FeedTbl();

                    if(selIndex < m_Table->GetItemCount())
                    {
                        m_Table->Select(selIndex);
                    }
                    else if(m_Table->GetItemCount() > 0)
                    {
                        m_Table->Select(m_Table->GetItemCount() - 1);
                    }
                }
            }
            else
            {
                wxMessageBox(_("This section doesn't exist !"), _("Section edit"), wxICON_EXCLAMATION, this);
            }
        }
    }
}

//------------------------------------------------------------------------------
void MainPanel::OnNameEdit(wxCommandEvent& event)
{
    std::vector<std::string> refList = GetSelectedRefList();
    if(refList.size() <= 0)
    {
        return;
    }

    // Items
    wxString newTxt = wxGetTextFromUser(_("Enter the new item name :"), _("Item edit"), StrStdToWx(m_IRDBManager->GetItemName(refList[0])), this);

    if(newTxt != _T(""))
    {
        for(int i = 0; i < refList.size(); i++)
        {
            m_IRDBManager->ItemDBSetName(refList[i], StrWxToStd(newTxt));
        }
        FeedTbl();
    }

    DeselectAll();
    for(int i = 0; i < refList.size(); i++)
    {
        m_Table->Select(m_Table->FindItem(-1, StrStdToWx(refList[i]))); // On reprend la selection...
    }
}
//------------------------------------------------------------------------------
void MainPanel::OnRefEdit(wxCommandEvent& event)
{
    std::vector<std::string> refList = GetSelectedRefList();
    if(refList.size() <= 0 || !RefWarningMsgBox())
    {
        return;
    }

    // Items
    wxString newTxt = wxGetTextFromUser(_("Enter the new reference :"), _("Item edit"), StrStdToWx(refList[0]), this);

    if(newTxt != _T(""))
    {
        if( ! m_IRDBManager->ItemExists( StrWxToStd(newTxt))) // Si la nouvelle ref n'existe pas...
        {
            for(int i = 0; i < refList.size(); i++)
            {
                if(i == 0) // Premier item ==> On donne la ref donnée
                {
                    m_IRDBManager->ItemDBSetRef(refList[i], StrWxToStd(newTxt));
                }
                else // On ajoute un numéro.
                {
                    wxString testNewRef = newTxt + wxString(_T("-")) + wxNbToStr(i + 1);
                    if( ! m_IRDBManager->ItemExists( StrWxToStd(testNewRef))) // Si la nouvelle ref n'existe pas...
                    {
                        m_IRDBManager->ItemDBSetRef(refList[i], StrWxToStd(testNewRef));
                    }
                }
            }
        }
        else
        {
            wxMessageBox(_("This item already exists !"), _("Item edit"), wxICON_EXCLAMATION, this);
        }
        FeedTbl();
    }

    DeselectAll();
    if(m_Table->GetItemCount() > 0)
    {
        m_Table->Select(0);
    }
}
//------------------------------------------------------------------------------
void MainPanel::OnPriceHTEdit(wxCommandEvent& event)
{
    std::vector<std::string> refList = GetSelectedRefList();
    if(refList.size() <= 0)
    {
        return;
    }

    // Items
    wxString newTxt = wxGetTextFromUser(_("Enter the new item price (HT) :"), _("Item edit"), wxNbToStr( m_IRDBManager->GetItemPriceHT(refList[0])) , this);
    double newNb = 0;
    if(newTxt != _T("") && wxStrToNb(newTxt, &newNb))
    {
        for(int i = 0; i < refList.size(); i++)
        {
            m_IRDBManager->ItemDBSetPriceHT(refList[i], newNb);
        }
        FeedTbl();
    }
    else if(newTxt != _T(""))
    {
        wxMessageBox(_("Please enter a valid number..."), _("Item edit"), wxICON_EXCLAMATION, this);
    }

    DeselectAll();
    for(int i = 0; i < refList.size(); i++)
    {
        m_Table->Select(m_Table->FindItem(-1, StrStdToWx(refList[i]))); // On reprend la selection...
    }
}
//------------------------------------------------------------------------------
void MainPanel::OnTaxeEdit(wxCommandEvent& event)
{
    std::vector<std::string> refList = GetSelectedRefList();
    if(refList.size() <= 0)
    {
        return;
    }

    // Items
    wxString newTxt = wxGetTextFromUser(_("Enter the new item taxe (%) :"), _("Item edit"), wxNbToStr( m_IRDBManager->GetItemTaxe(refList[0])) , this);
    double newNb = 0;
    if(newTxt != _T("") && wxStrToNb(newTxt, &newNb))
    {
        for(int i = 0; i < refList.size(); i++)
        {
            m_IRDBManager->ItemDBSetTaxe(refList[i], newNb);
        }
        FeedTbl();
    }
    else if(newTxt != _T(""))
    {
        wxMessageBox(_("Please enter a valid number..."), _("Item edit"), wxICON_EXCLAMATION, this);
    }

    DeselectAll();
    for(int i = 0; i < refList.size(); i++)
    {
        m_Table->Select(m_Table->FindItem(-1, StrStdToWx(refList[i]))); // On reprend la selection...
    }
}
//------------------------------------------------------------------------------
void MainPanel::OnEcoPartEdit(wxCommandEvent& event)
{
    std::vector<std::string> refList = GetSelectedRefList();
    if(refList.size() <= 0)
    {
        return;
    }

    // Items
    wxString newTxt = wxGetTextFromUser(_("Enter the new item Eco Part :"), _("Item edit"), wxNbToStr( m_IRDBManager->GetItemEcoPart(refList[0])) , this);
    double newNb = 0;
    if(newTxt != _T("") && wxStrToNb(newTxt, &newNb))
    {
        for(int i = 0; i < refList.size(); i++)
        {
            m_IRDBManager->ItemDBSetEcoPart(refList[i], newNb);
        }
        FeedTbl();
    }
    else if(newTxt != _T(""))
    {
        wxMessageBox(_("Please enter a valid number..."), _("Item edit"), wxICON_EXCLAMATION, this);
    }

    DeselectAll();
    for(int i = 0; i < refList.size(); i++)
    {
        m_Table->Select(m_Table->FindItem(-1, StrStdToWx(refList[i]))); // On reprend la selection...
    }
}
//------------------------------------------------------------------------------
void MainPanel::OnSearchText(wxCommandEvent& event)
{
    FeedTbl();

    // Couleur de fond du texte rouge si rien trouvé, vert sinon.
    if( m_TextSearch->GetValue() == _T(""))
    {
        m_TextSearch->SetBackgroundColour(*wxWHITE);
    }
    else if( m_Table->GetItemCount() > 0 )
    {
        m_TextSearch->SetBackgroundColour(*wxGREEN);
    }
    else
    {
        m_TextSearch->SetBackgroundColour(*wxRED);
    }

    m_TextSearch->Refresh();
}
//------------------------------------------------------------------------------
bool MainPanel::RefWarningMsgBox()
{
    return (wxMessageBox(_("Warning : Every items with the current reference will be removed from estimates !\nDo you want to continue ?"), _("Item edit"), wxICON_WARNING | wxYES_NO | wxNO_DEFAULT, this) == wxYES);
}
//------------------------------------------------------------------------------
void MainPanel::FeedTbl()
{
    wxWindowUpdateLocker winLock(m_Table); // Bloque les redraw durant son temps de vie.

    // On prend la ref du dernier séléctioner pour le reséléctioonner, si il est toujours là.
    wxString selItemRef = _T("");
    if(m_Table->GetFirstSelected() != -1)
    {
        selItemRef = m_Table->GetItemText(m_Table->GetFirstSelected());
    }

    m_Table->ClearAll();

    wxString searchStr = m_TextSearch->GetValue().Lower(); // TT en lower ==> Case insensitive

    if(m_isItems) // Si on traite les items dans ce panel
    {
        m_Table->InsertColumn(0, _("Reference"));
        m_Table->InsertColumn(1, _("Name"));
        m_Table->InsertColumn(2, _("Price"));
        m_Table->InsertColumn(3, _("Taxe"));
        m_Table->InsertColumn(4, _("Eco Part"));
    }
    else
    {
        m_Table->InsertColumn(0, _("Section"));
    }

    if(m_isItems) // Si on traite les items dans ce panel
    {
        std::vector<std::string> itemList = m_IRDBManager->GetAllItemsList();

        for(int i = 0; i < itemList.size(); i++)
        {
            wxString itemRefwxStr = StrStdToWx(itemList[i]);
            wxString itemNamewxStr = StrStdToWx(m_IRDBManager->GetItemName(itemList[i]));

            if(itemRefwxStr.Lower().Find(searchStr) != wxNOT_FOUND || itemNamewxStr.Lower().Find(searchStr)  != wxNOT_FOUND) // Si trouvé...
            {
                long tmp = m_Table->InsertItem(i, itemRefwxStr);

                m_Table->SetItem(tmp, 1, itemNamewxStr);

                m_Table->SetItem(tmp, 2, wxNbToStr(m_IRDBManager->GetItemPriceHT(itemList[i])));

                m_Table->SetItem(tmp, 3, wxNbToStr(m_IRDBManager->GetItemTaxe(itemList[i])));

                m_Table->SetItem(tmp, 4, wxNbToStr(m_IRDBManager->GetItemEcoPart(itemList[i])));
            }
        }
    }
    else
    {
        std::vector<std::string> sectionList = m_IRDBManager->GetAllSectionsList();

        for(int i = 0; i < sectionList.size(); i++)
        {
            wxString sectionNamewxStr = StrStdToWx(sectionList[i]);

            if(sectionNamewxStr.Lower().Find(searchStr) != wxNOT_FOUND) // Si trouvé...
            {
                m_Table->InsertItem(i, sectionNamewxStr);
            }
        }
    }

    if(m_isItems) // Si on traite les items dans ce panel
    {
        m_Table->SetColumnWidth(0, wxLIST_AUTOSIZE);
        m_Table->SetColumnWidth(1, wxLIST_AUTOSIZE);
        m_Table->SetColumnWidth(2, wxLIST_AUTOSIZE);
        m_Table->SetColumnWidth(3, wxLIST_AUTOSIZE);
        m_Table->SetColumnWidth(4, wxLIST_AUTOSIZE);
    }
    else
    {
        m_Table->SetColumnWidth(0, wxLIST_AUTOSIZE);
    }

    // Special : selectionne la nouvelle ref (a changé de place...)
    if(selItemRef != _T("") && m_Table->FindItem(-1, selItemRef) != -1)
    {
        m_Table->Select(m_Table->FindItem(-1, selItemRef));
    }
    else if(m_Table->GetItemCount() > 0)
    {
        m_Table->Select(0);
    }

    // Il faut aussi penser à rafraichir le menu...
    static_cast<MainFrame*>(GetParent()->GetParent())->MyRefreshMenu();
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//-                               PanelSections                                   -
//------------------------------------------------------------------------------
/*PanelSections::PanelSections( wxWindow* parent, wxWindowID id, DevisManager *devisManager, const wxPoint& pos, const wxSize& size, long style ) : wxPanel( parent, id, pos, size, style )
{
    // Devis Manager
    m_DevisManager = devisManager;

	// Controls
    wxBoxSizer *MainSizer = new wxBoxSizer( wxHORIZONTAL );

	SetSizer(MainSizer);
}*/
